#include "FluidColorMatrix.h"

void FluidColorMatrix::initialize() {
  uint8_t step = 0;
  switch(resolution) {
    case FluidColorResolution::FULL_FLUIDCOLOR:
      numColors = 255;
      step = 1;
      break;
    case FluidColorResolution::HALF_FLUIDCOLOR:
      numColors = 128;
      step = 2;
      break;
    case FluidColorResolution::QUATER_FLUIDCOLOR:
      numColors = 64;
      step = 4;
      break;
    case FluidColorResolution::EIGHTH_FLUIDCOLOR:
    default:
      numColors = 32;
      step = 8;
      break;
  }

  colors = new CRGB[numColors];

  for(uint16_t i = 0; i < numColors; i+=step) {
    colors[i/step] = CRGB(CHSV(i, 255, 255));
  }
}

void FluidColorMatrix::execute() {
  switch(type) {
    case FluidColorType::PIXEL_BY_PIXEL_FLUIDCOLOR:
      for(int16_t x = 0; x < matrix->getWidth(); x++) {
        for(int16_t y = 0; y < matrix->getHeight(); y++) {
          matrix->drawPixel(x, y, 
            colors[((matrix->getWidth() * y + x) + colorShifter) % numColors]);
        }
      }
      break;
    case FluidColorType::HORIZONTAL_FLUIDCOLOR:
      for(int16_t y = 0; y < matrix->getHeight(); y++) {
        matrix->drawFastHLine(0, y, matrix->getWidth(), 
          colors[(y + colorShifter) % numColors]);
      }
      break;
    case FluidColorType::VERTICAL_FLUIDCOLOR:
    default:
      for(int16_t x = 0; x < matrix->getWidth(); x++) {
        matrix->drawFastVLine(x, 0, matrix->getHeight(), 
          colors[(x + colorShifter) % numColors]);
      }
      break;
  }

  colorShifter++;
  matrix->requestShow();
}